@@ -8,6 +8,8 @@ http://opensource.org/licenses/MIT>, at your option. This file may not be
 copied, modified, or distributed except according to those terms.
 */
 
+//! Provides the rendering engine for topics and posts using [`AppConfig`], [`Tera`], and [`pulldown_cmark`].
+
 use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
@@ -20,8 +22,14 @@ use pulldown_cmark::{Parser, html};
 use tera::Tera;
 use tera::Context as TemplateContext;
 
+/// Static defaults for the rendering engine.
 mod default;
 
+
+/// Rendering engine for topics and posts.
+///
+/// [`Engine`] stores an [`Arc<AppConfig>`] and a [`Tera`] instance from which
+/// all rendering and serving tasks are executed.
 #[derive(Debug)]
 pub(crate) struct Engine {
     pub app: Arc<AppConfig>,
@@ -29,6 +37,7 @@ pub(crate) struct Engine {
 }
 
 impl Engine {
+    /// Creates a new [`Engine`] from a given [`AppConfig`].
     pub(crate) fn new(app: Arc<AppConfig>) -> Engine {
 	trace!("Loading rendering engine");
 	let instance = Self::load_template(app.clone()).unwrap();
@@ -57,13 +66,22 @@ impl Engine {
 	Ok(tera)
     }
 
+    /// Renders `/:topic` content as HTML
     pub(crate) fn render_topic(&self, topic_slug: &str) -> Result<String> {
-	debug!("Rendering topic: '{}'", topic_slug);
 	let site = &self.app.site;
-	let topic_data = self.load_topic(topic_slug)?;
 	let mut context = TemplateContext::new();
 	context.insert("site", site);
-	context.insert("posts", &topic_data);
+
+	if topic_slug == "gallery" {
+	    debug!("Rendering image gallery");
+	    let gallery = self.load_gallery()?;
+	    context.insert("gallery", &gallery);
+	} else {
+	    debug!("Rendering topic: '{}'", topic_slug);
+	    let topic_data = self.load_topic(topic_slug)?;
+	    context.insert("posts", &topic_data);
+	}
+
 	let output = self.instance.render(&site.template, &context)
 	    .with_context(|| format!("failed rendering topic: {}, with Tera instance: {:?}", topic_slug, self.instance))?;
 
@@ -95,6 +113,21 @@ impl Engine {
 	Ok(contents)
     }
 
+    fn load_gallery(&self) -> Result<Vec<PathBuf>> {
+	debug!("Loading gallery content");
+	let gallery_path = Path::new(&self.app.docpaths.webroot).join("gallery").join("ext");
+	let pat = format!("{}/*.jpg", gallery_path.display());
+	let mut paths = common::path_matches(&pat)?;
+	
+	trace!("Stripping path prefixes");
+	for path in paths.iter_mut() {
+	    *path = path.strip_prefix(&self.app.docpaths.webroot)?.to_path_buf();
+	}
+
+	Ok(paths)
+    }
+    
+    /// Renders `/:topic/posts/:post` content as HTML
     pub(crate) fn render_post(&self, topic_slug: &str, post: &str) -> Result<String> {
 	debug!("Rendering post: '{}'", post);
 	let site = &self.app.site;
@@ -220,4 +253,25 @@ Super Wow!
 
 	assert!(page.contains("Coming Soon"));
     }
+    
+    #[test]
+    fn check_render_gallery_topic() {
+	let dir = tempfile::tempdir().unwrap();
+	let mut src: &[u8] = b"Site Name\nAuthor Name\nOne, Gallery\nadmin\n";
+	let config = AppConfig::generate(&dir, &mut src).unwrap();
+	let config = Arc::new(config);
+	let engine = Engine::new(config);
+
+	let fake_img = "some bytes";
+	let fake_img_2 = "some more bytes";
+	
+	let mut f = File::create(&dir.path().join("site/webroot/gallery/ext/0.jpg")).unwrap();
+	f.write_all(&fake_img.as_bytes()).unwrap();
+
+	let mut f = File::create(&dir.path().join("site/webroot/gallery/ext/1.jpg")).unwrap();
+	f.write_all(&fake_img_2.as_bytes()).unwrap();
+	let page = engine.render_topic("gallery").unwrap();
+
+	assert!(page.contains("<script>"));
+    }
 }
